#ifndef AABB_HXX
#define AABB_HXX

#include "Primitive.hxx"

class AABB : public Primitive
{
    Vec3f _center;
    Vec3f _min;
    Vec3f _max;

    Box CalcBounds()
    {
        Box bb;
        bb.Extend(_min);
        bb.Extend(_max);
        return bb;
    }

public:
  AABB(Vec3f min, Vec3f max) : _min(min), _max(max)
  {
     // compute center of the axis alignet bounding box
     _center = (_min + _max) * 0.5;
  };

  ~AABB()
  {};

  bool Intersect(Ray &ray)
  {
    float tmin, tmax, tymin, tymax, tzmin, tzmax;

    // inverse direction to catch float problems
    Vec3f invrd = 1.0 / ray.dir;

    if (invrd.x >= 0)
    {
        tmin = (_min.x - ray.org.x) * invrd.x;
        tmax = (_max.x - ray.org.x) * invrd.x;
    }else
    {
        tmin = (_max.x - ray.org.x) * invrd.x;
        tmax = (_min.x - ray.org.x) * invrd.x;
    }

    if (invrd.y >= 0)
    {
        tymin = (_min.y - ray.org.y) * invrd.y;
        tymax = (_max.y - ray.org.y) * invrd.y;
    }else
    {
        tymin = (_max.y - ray.org.y) * invrd.y;
        tymax = (_min.y - ray.org.y) * invrd.y;
    }

    if ( (tmin > tymax) || (tymin > tmax) ) return false;
    if ( tymin > tmin) tmin = tymin;
    if ( tymax < tmax) tmax = tymax;

    
    if (invrd.z >= 0)
    {
        tzmin = (_min.z - ray.org.z) * invrd.z;
        tzmax = (_max.z - ray.org.z) * invrd.z;
    }else
    {
        tzmin = (_max.z - ray.org.z) * invrd.z;
        tzmax = (_min.z - ray.org.z) * invrd.z;
    }

    if ( (tmin > tzmax) || (tzmin > tmax) ) return false;
    if ( tzmin > tmin) tmin = tzmin;
    if ( tzmax < tmax) tmax = tzmax;

    // check if values are valid
    if (tmin < 0) tmin = tmax;
    if (tmax < 0) return false;

    ray.t = tmin;
    return true;
  };
};

#endif
