#ifndef CYLINDRICCAMERA_HXX
#define CYLINDRICCAMERA_HXX

#include "Camera.hxx"


class CylindricCamera : public Camera
{
  // input values
  Vec3f pos,up;
  float height;
  float radius;

  float invResX;
  float invResY;
  
public:
  CylindricCamera(Vec3f pos,Vec3f up,
		    float height, float radius,
		    int resX, int resY
		    )
    : Camera(resX,resY),pos(pos),up(up),height(height),radius(radius)
  {
     invResX = 1.0 / float(resX);
     invResY = 1.0 / float(resY); 
  }

  virtual ~CylindricCamera()
  {}

  
  virtual bool InitRay(float x, float y, Ray &ray)
  {
    // precompute angles
    float alpha = x * invResX * 2.0 * M_PI;

    // compute ray direction
    Vec3f p;
    p.x = cos(alpha) * radius;
    p.y = height * 0.5 - y * invResY;
    p.z = sin(alpha) * radius;

    // set position and normalized direction
    ray.org = pos + p;
    ray.dir = -p / Length(p);
    ray.t = Infinity;
    
    return true;
  };

};

#endif
