#ifndef FLATSHADERTRANSPARENT_HXX
#define FLATSHADERTRANSPARENT_HXX

#include "Shader.hxx"

class FlatTransparentShader : public Shader
{
public:
    Vec3f color;

    FlatTransparentShader(Scene *scene, Vec3f color=Vec3f(1.0,1.0,1.0))
        : Shader(scene), color(color)
    {};

    Vec3f Shade(Ray &ray)
    {     
        // results
        Vec4f result = Vec4f(0,0,0,0);

        // get uv values
        Vec2f uv = ray.hit->GetUV(ray);

        // for each assigned texture
        for (unsigned int i=0; i < getTextureCount(); i++)
        {
            // get texture value
            const Vec4f& tex = getTexture(i)->GetTexel(uv);
            
            // combine texture colors by blending them together
            result = lerp(result, tex, tex.w());
        }

        // if pixel is transparent, then we have to trace the ray back into the scene
        if (result.w() < 1)
        {
            // create new ray which could be ray traced back into the scene
            Ray r = ray;
    
            // the ray should move from the shaded surface
            r.org = ray.org + ray.t * ray.dir;
            r.hit = NULL;
            r.t = Infinity;

            // trace ray back into the scene and get the color
            Vec3f c = scene->RayTrace(r);

            // combine result witht the texture color
            // here traced color is our destination color and texture is a source color
            result = Vec4f(lerp(c, result.xyz(), result.w()), 1);
        }

        // return result value multiplied by the flat shaded color 
        return result.xyz() * color;
    };
};

#endif
