#ifndef IMAGE_HXX
#define IMAGE_HXX

#include "Vec4f.hxx"

class Image
{
    int resX, resY;
    Vec4f *pixel;

public:
    Image(int resX,int resY) : resX(resX),resY(resY)
    {
        assert (resX > 0 && resY > 0);
        pixel = new Vec4f[resX*resY];
    }

    Image() : resX(0),resY(0)
    {
        pixel = NULL;
    }

    ~Image()
    { 
        if (pixel) delete[] pixel;
    };
    
    Vec4f *operator[](int y)
    { 
        assert(pixel != NULL);
        return pixel + y * resX; 
    };

    //! get image dimansion x
    int getWidth() const { return resX; }

    //! get image dimansion y
    int getHeight() const { return resY; }

    //! Get pixel value on certain position
    Vec4f& getPixel (int x, int y)
    { 
        if (x < 0 || y < 0) printf("x=%d, y=%d\n", x, y);
        // just for sure
        assert(pixel != NULL && x >=0 && y >= 0);
        assert(x < resX && y < resY);

        // return pixel
        return pixel[y * resX + x]; 
    };

    //! Set pixel on certain position
    void setPixel (const Vec4f& p, int x, int y)
    { 
        // just for sure
        assert(pixel != NULL && x >=0 && y >= 0);
        assert(x < resX && y < resY);

        // return pixel
        pixel[y * resX + x] = p; 
    };
    void setPixel (const Vec3f& p, int x, int y)
    { 
        // just for sure
        assert(pixel != NULL && x >=0 && y >= 0);
        assert(x < resX && y < resY);

        // return pixel
        pixel[y * resX + x] = p; 
    };
    
    //! Write content of the image as ppm file     
    void WritePPM(char *fileName);

    //! Read image from ppm file
    bool ReadPPM(char *fileName);

    /**
     * Read image from png file 
     * The image will be converted to float r,g,b
     **/
    bool ReadPNG(char* fileName);

    /**
     * Write image content as RGB png file.
     **/
    void WritePNG(char* fileName);

    /**
     * Load image from file. This function will check the 
     * file extension and use appropriate function
     **/
    bool LoadImage(char* fileName);

};

#endif
