#ifndef INFINITEPLANE_HXX
#define INFINITEPLANE_HXX

class InfinitePlane : public Primitive
{
    Vec3f normal;
    Vec3f origin;

    Box CalcBounds()
    {
        // a bounding box for an infinite plane 
        // does not make any sense. However, since we could
        // still use this class, we have to decide what to return here.
        // This should be a plausible decision. I would suggest to 
        // return box of 0 dimension.
        Box b = Box();
        b.Extend(Vec3f(0,0,0));
        return b;

        // any plausible solution is correct, since we will not
        // use this class in the future
    }

public:

    InfinitePlane(Vec3f origin, Vec3f normal)
    : normal(normal),origin(origin)
    { Normalize(normal); };
    
    ~InfinitePlane()
    {}

    Vec3f GetNormal(Ray &ray) { return normal; }
    
    bool Intersect(Ray &ray)
    {
        Vec3f diff = origin - ray.org;
        float t = Dot(diff,normal) / Dot(ray.dir,normal);
        if (t < Epsilon || t > ray.t) return false;
        ray.t = t;
        return true;
    };

};

#endif
