#ifndef LIGHT_HXX
#define LIGHT_HXX

#include "Vec3f.hxx"
#include "Ray.hxx"

class Scene; // forward declaration

class Light
{
protected:

    //! it is good to have friends ;-)
    friend class Scene;

    //! Light is in this scene
    Scene* mScene;

public:
    Light()
    {
        mScene = NULL;
    };
    
    virtual ~Light() {}

    /**
     * Illuminate the light. The ray will be changed into a ray
     * pointing to the light source, hence we call it shadow ray.
     * @param ray Here a new ray (shadow ray) will be stored
     * @param intensity The intensity of the ray (i.e. color) will be given back here
     * @param index Specify index value of the light ray. The value should be in the
     * range [0; GetNumberOfRays). 
     **/
    virtual bool Illuminate(Ray &ray, Vec3f &intensity, unsigned int index = 0) = 0;

    /**
     * Return number of rays required to accomplish 
     * the shading produced by this light. (i.e. Point light = 1 ray)
     * area light = n rays)
     **/
    virtual unsigned int GetNumberOfRays() const { return 1; }

};

#endif
