#ifndef MIRROR_SHADER_HXX
#define MIRROR_SHADER_HXX

#include "Shader.hxx"

class MirrorShader : public Shader
{
private:
    Vec3f reflection;

public:
    MirrorShader(Scene *scene, const Vec3f& reflection) : Shader(scene), reflection(reflection)
    {};
    
    ~MirrorShader() {}

    //! Mirror the ray on the surface and trace back
    Vec3f Shade(Ray &ray)
    {
        // get normal of the hit surface 
        Vec3f normal = ray.hit->GetNormal(ray);

        // compute reflection vector
        Vec3f reflect = ray.dir - 2*Dot(normal,ray.dir)*normal;
        
        // change ray direction and position
        ray.org = ray.org + ray.t * ray.dir;
        ray.dir = reflect;
        ray.t = Infinity;
        ray.hit = NULL;
        ray.u = ray.v = 0.0;

        // trace the ray back into the scene 
        return reflection * scene->RayTrace(ray);
    }
};

#endif
