#include "stdafx.h"

#include "Object.hxx"

//--------------------------------------------------------------------------------
Vec3f vec3fFromStream(std::istream &aStream)
{
    float x, y, z;
    aStream >> x >> std::ws >> y >> std::ws >> z;
    return Vec3f(x, y, z);
}

//--------------------------------------------------------------------------------
Vec2f vec2fFromStream(std::istream &aStream)
{
    float x, y;
    aStream >> x >> std::ws >> y;
    return Vec2f(x, y);
}

//--------------------------------------------------------------------------------
struct FaceIdcs 
{
    int v[4], vn[4], vt[4];
    FaceIdcs()
    { memset(this, 0, sizeof(*this)); }

    FaceIdcs(std::istream &aStream)
    { 
        memset(this, 0, sizeof(*this));
        char c;

        for(int i = 0; i < 3; i++)
        {
            aStream >> std::ws >> v[i] >> std::ws;
            if(aStream.peek() != '/')
                continue;
            aStream >> c >> std::ws;
            if(aStream.peek() == '/')
            {
                aStream >> c >> std::ws >> vn[i];
                continue;
            }else
                aStream >> vt[i];
            if(aStream.peek() != '/')
                continue;
            aStream >> c >> vn[i];
        }
    }
};


//--------------------------------------------------------------------------------
void Object::ParseOBJ(char *fileName, const Object::PrimitiveFactory& factory)
{
    std::cout << "Parsing OBJFile : " << fileName << std::endl;

    std::ifstream ifile(fileName);

    std::vector<FaceIdcs> faces;
    std::vector<Vec3f> vert, norm;
    std::vector<Vec2f> text;

    // while not end of file
    while(!ifile.eof())
    {
        // get line
        std::string curLine;
        std::getline(ifile, curLine);

        // read type of the line
        std::istringstream issLine(curLine);
        std::string lineType;
        issLine >> std::ws >> lineType;

        // based on the type parse data
        if(lineType == "v")
            vert.push_back(vec3fFromStream(issLine));
        else if (lineType == "vn")
            norm.push_back(vec3fFromStream(issLine));
        else if (lineType == "vt")
            text.push_back(vec2fFromStream(issLine));
        else if (lineType == "f")
            faces.push_back(issLine);
    }

    // iterate through all stored faces and create triangles
    for(std::vector<FaceIdcs>::iterator it = faces.begin(); it != faces.end(); it++)
    {
        Vec3f v[3], vn[3];
        Vec2f vt[3];
        for(int i = 0; i < 3; i++)
        {
            v[i] = vert[it->v[i] - 1];
            if(it->vn[i] >= 1 && it->vn[i] <= (int)norm.size())
                vn[i] = norm[it->vn[i] - 1];
            if(it->vt[i] >= 1 && it->vt[i] <= (int)text.size())
                vt[i] = text[it->vt[i] - 1];
        }

        // create primitive by the factory 
        Add (factory.Create(v[0], v[1], v[2], vn[0], vn[1], vn[2], vt[0], vt[1], vt[2]));
    }

    // some debug
    //std::cout << "Number of Vertices: " << (vert.size() / 3) << std::endl;
    //std::cout << "Number of Triangles: " << primitive.size() << std::endl;
    //std::cout << "Finished Parsing" << std::endl;

}
