#ifndef POINTLIGHT_HXX
#define POINTLIGHT_HXX

#include "Light.hxx"

class PointLight : public Light
{
private:
    Vec3f position;  // origin
    Vec3f intensity; // emission (red, green, blue)

public:
    
    /**
     * Inititalize light source.
     * @param position Position of the point light 
     * @param intensity Intensity of the light source (color)
     **/
    PointLight(Vec3f position, Vec3f intensity)
    : Light(),position(position),intensity(intensity)
    {

    };  
    
    /**
     * Illuminate a ray by this light source.
     * The illumination is done by computing the attenuation factor
     * which is based on the distance to the surface point and the light source.
     **/
    bool Illuminate(Ray &ray, Vec3f &intensity, unsigned int index = 0)
    {
        // ray points towards point light position from the surface
        ray.dir = position - ray.org;
        ray.hit = NULL;
        ray.t = Length(ray.dir) - Epsilon; // maximal ray length
        Normalize(ray.dir);
        
        // compute linear attenuation
        // see http://www.gamasutra.com/features/20050729/lacroix_01.shtml        
        float att = 1.0f / (ray.t * ray.t); // linear attenuation
        intensity = att * this->intensity;
        return true;
    };
};

#endif
