#ifndef PRIMITIVE_HXX
#define PRIMITIVE_HXX

#include "Ray.hxx"
#include "Box.hxx"
#include "Shader.hxx"

class Primitive
{
protected:
    Box bounds;
    bool bDirtyBounds;
    Shader* shader;
    bool bCastShadows;

    virtual Box CalcBounds() = 0;
    
public:
    Primitive()
    {
        bDirtyBounds = true;
        shader = NULL;
        bCastShadows = true;
    }
    
    virtual ~Primitive(){}

    virtual bool Intersect(Ray &ray) = 0;
    virtual bool Occluded(Ray &ray) {return Intersect(ray);}
    virtual Vec3f GetNormal(Ray &ray) = 0;
    virtual Vec2f GetUV(Ray &ray) { return Vec2f(); }

    Shader* getShader() { return shader; }
    void setShader(Shader* sh) { shader = sh; }
    
    //! Get bounds of this box
    inline const Box& GetBounds()
    {
        if (bDirtyBounds)
        {
            bounds = CalcBounds();
            bDirtyBounds = false;
        }
        return bounds;
    }
    
    //! get direct access to the cast shadow variable
    inline bool& castShadows() { return bCastShadows; }

};

#endif
