#ifndef SHADER_HXX
#define SHADER_HXX

#include "Vec3f.hxx"
#include "Vec4f.hxx"
#include "Ray.hxx"
#include "Texture.hxx"

class Scene; // forward declaration

class Shader
{
protected:

    //! Scene pointer to retrieve data
	Scene *scene;

    //! List of textures used in this shader
    std::vector<Texture*> mTexture;

public:
    Shader(Scene *scene) : scene(scene)
    {};
    
    virtual ~Shader() {}

    virtual Vec3f Shade(Ray &ray) = 0;

    //! Get number of textures used by the shader
    unsigned int getTextureCount() const { return mTexture.size(); }

    //! Add new texture to the shader 
    void addTexture(Texture* tex) 
    {
        mTexture.push_back(tex);
    }

    //! Remove texture from the shader (Note: Pointer is not deleted)
    void removeTexture(Texture* tex)
    {
        for (std::vector<Texture*>::iterator it=mTexture.begin(); it != mTexture.end(); )
            if ((*it) == tex) {
                it = mTexture.erase(it);
            }else
                it++;
    }

    //! Get texture according to the index (start from 0)
    Texture* getTexture(unsigned int i)
    {
        if (i >= getTextureCount()) return NULL;
        return mTexture[i];
    }

};

#endif
