#ifndef SMOOTH_TRIANGLE_HXX
#define SMOOTH_TRIANGLE_HXX

#include "Triangle.hxx"

class SmoothTriangle : public Triangle
{
private:

    Vec3f na, nb, nc;

public:

    //! Constructor
    SmoothTriangle(Vec3f a, Vec3f b, Vec3f c)
        : Triangle(a,b,c)
    {};

    //! Constructor
    SmoothTriangle(Vec3f a, Vec3f b, Vec3f c, Vec3f na, Vec3f nb, Vec3f nc)
        : Triangle(a,b,c),na(na), nb(nb), nc(nc)
    {};

    //! Destructor
    virtual ~SmoothTriangle()
    {
    }

    //! Set vertex normals
    void SetNormals(const Vec3f& _na, const Vec3f& _nb, const Vec3f& _nc)
    {
        na = _na;
        nb = _nb;
        nc = _nc;

        Normalize(na);
        Normalize(nb);
        Normalize(nc);
    }

    //! Get normal, but this time it is a smooth normal
    Vec3f GetNormal(Ray &ray)
    {
        // assume u/v coordinates in ray correspond to beta(u) and gamma
        // (v) barycentric coordinates of hit point on triangle (have to be
        // stored like this in the intersection code !)

        Vec3f normal = ray.u * nb + ray.v * nc + (1 - ray.u - ray.v) * na;
        Normalize(normal);

        return normal;
    };

};

#endif
