#ifndef TEXTURE_HXX
#define TEXTURE_HXX

#include "Image.hxx"
#include "Vec2f.hxx"

class Texture
{
public:

    //! Texture does support following texel coordinate wrapping modes
    enum Mode{
        //! clamp texture coordinates
        CLAMP = 0,

        //! repeat texture coordinates
        REPEAT = 1,

        //! Clamp ot border
        CLAMP_TO_BORDER = 2
    };

    //! Sampling mode 
    enum InterpolationMode
    {
        //! no interpolation, hence sample the nearest texel
        NEAREST = 0,

        //! Linear interpolation
        LINEAR = 1
    };

    //! Create empy texture object
    Texture() : mMode(CLAMP), mInterpolationMode(NEAREST)
    {
    };
        
    //! Release used data
    virtual ~Texture()
    {
    }

    //! Get current texture mode 
    Mode getWrapMode() const { return mMode; }

    //! Set current texture mode 
    void setWrapMode(Mode mode) { mMode = mode; }

    //! Get interpolation mode 
    InterpolationMode getInterpolationMode() const { return mInterpolationMode; }

    //! Set interpolation mode 
    void setInterpolationMode(InterpolationMode mode) { mInterpolationMode = mode; }

    /**
     * Get texel on certain position.
     * Here the correct wrapping mode should be used. (based on getWrapMode() method)
     * Interpolate texels if required. (based on getInterpolationMode() method)
     **/
    virtual Vec4f GetTexel(float u, float v) = 0;
    inline  Vec4f GetTexel(const Vec2f& uv) { return GetTexel(uv.x(), uv.y()); }

    //! Get border color 
    inline const Vec4f& getBorderColor() const { return mBorderColor; }

    //! Set border color 
    inline void setBorderColor(const Vec4f& c) { mBorderColor = c; }

protected:

    //! current mode used to handle texture coordinates
    Mode mMode;

    //! current used interpolation mode 
    InterpolationMode mInterpolationMode;

    //! Border color
    Vec4f mBorderColor;

};

#endif
