#ifndef TEXTURED_SMOOTH_TRIANGLE_HXX
#define TEXTURED_SMOOTH_TRIANGLE_HXX

#include "SmoothTriangle.hxx"
#include "Vec2f.hxx"

class TexturedSmoothTriangle : public SmoothTriangle
{
private:

    Vec2f ta, tb, tc;

public:

    //! Constructor
    TexturedSmoothTriangle(Vec3f a, Vec3f b, Vec3f c)
        : SmoothTriangle(a,b,c)
    {};

    //! Constructor
    TexturedSmoothTriangle(Vec3f a, Vec3f b, Vec3f c, Vec3f na, Vec3f nb, Vec3f nc)
        : SmoothTriangle(a,b,c,na,nb,nc)
    {};

    //! Constructor
    TexturedSmoothTriangle(Vec3f a, Vec3f b, Vec3f c, Vec3f na, Vec3f nb, Vec3f nc, Vec2f ta, Vec2f tb, Vec2f tc)
        : SmoothTriangle(a,b,c,na,nb,nc), ta(ta), tb(tb), tc(tc)
    {};

    //! Set vertex UV coordinates
    void SetUV(const Vec2f& _ta, const Vec2f& _tb, const Vec2f& _tc)
    {
        ta = _ta;
        tb = _tb;
        tc = _tc;
    }

    //! Get normal, but this time it is a smooth normal
    Vec2f GetUV(Ray &ray)
    {
        // assume u/v coordinates in ray correspond to beta(u) and gamma
        // (v) barycentric coordinates of hitpoint on triangle (have to be
        // stored like this in the intersection code !)
        float u = ray.u * tb.x() + ray.v * tc.x() + (1 - ray.u - ray.v) * ta.x();
        float v = ray.u * tb.y() + ray.v * tc.y() + (1 - ray.u - ray.v) * ta.y();

        return Vec2f(u,v);
    };

};

#endif
