#ifndef TRIANGLE_HXX
#define TRIANGLE_HXX

#include "Primitive.hxx"

class Triangle : public Primitive
{
private:

    Vec3f a, b, c;

    Box CalcBounds()
    {
        Box bounds_;
        
        bounds_.Extend(a);
        bounds_.Extend(b);
        bounds_.Extend(c);

        return bounds_;
    }

public:

    //! Constructor
    Triangle(Vec3f a, Vec3f b, Vec3f c)
        : Primitive(),a(a),b(b),c(c)
    {};

    //! Destructor
    virtual ~Triangle()
    {
    }
    
    //! Compute ray-triangle intersection
    bool Intersect(Ray &ray)
    {
        const Vec3f edge1 = b-a;
        const Vec3f edge2 = c-a;

        const Vec3f pvec = ray.dir^edge2;

        const float det = Dot(edge1, pvec);
        if (fabs(det) < Epsilon) return false;

        const float inv_det = 1.0f / det;

        const Vec3f tvec = ray.org-a;
        float lambda = Dot(tvec, pvec);
        lambda *= inv_det;

        if (lambda < 0.0f || lambda > 1.0f) return false;

        const Vec3f qvec = tvec^edge1;
        float mue = Dot(ray.dir, qvec);
        mue *= inv_det;

        if (mue < 0.0f || mue+lambda > 1.0f) return false;

        float f = Dot(edge2, qvec);
        f = f * inv_det - Epsilon;
        if (ray.t <= f || f <  Epsilon  ) return false;

        ray.u = lambda;
        ray.v = mue;
        ray.t = f;
        ray.hit = this;

        return true;
    };

    //! Get triangle's normal
    virtual Vec3f GetNormal(Ray &ray)
    {
        Vec3f e1 = b - a;
        Vec3f e2 = c - a;
        Vec3f normal = Cross(e1,e2);
        Normalize(normal);
        return normal;
    };

};

#endif
