#include "stdafx.h"
#include "Vec2f.hxx"
#include "Vec3f.hxx"
#include "Vec4f.hxx"

float deg2rad(float f) { return  f * (float)M_PI / 180.0f; }
float rad2deg(float f) { return  f * 180.0f / (float)M_PI; }

double drand48(void)
{
	unsigned int x = (rand() << 16) | rand();
	return ((double)x / (double)0xffffffff);
}

float frand()
{
    unsigned int x = rand();
    return float(float(x) / float(RAND_MAX));
}

float lerp(float a, float b, float f)
{
    return a * (1-f) + b * f;
}

Vec2f lerp(const Vec2f& a, const Vec2f& b, float f)
{
    return Vec2f(a.x() * (1-f) + b.x() * f,
                 a.y() * (1-f) + b.y() * f);
}

Vec3f lerp(const Vec3f& a, const Vec3f& b, float f)
{
    return Vec3f(a.x() * (1-f) + b.x() * f,
                 a.y() * (1-f) + b.y() * f,
                 a.z() * (1-f) + b.z() * f);
}

Vec4f lerp(const Vec4f& a, const Vec4f& b, float f)
{
    return Vec4f(a.x() * (1-f) + b.x() * f,
                 a.y() * (1-f) + b.y() * f,
                 a.z() * (1-f) + b.z() * f,
                 a.w() * (1-f) + b.w() * f);
}


float cosrp(float a, float b, float x)
{
    float ft = x * 3.1415927;
    float f = (1 - cos(ft)) * 0.5;
    return  a*(1-f) + b*f;
}

float Noise2D(int x, int y)
{
    int n = x + y * 57;
    n = (n<<13) ^ n;
    float r = ( 1.0 - ( (n * (n * n * 15731 + 789221) + 1376312589) & 0x7fffffff) / 1073741824.0);;
    return r;
}

float SmoothNoise2D(float x, float y)
{
    double n1 = Noise2D((int)x, (int)y);
    double n2 = Noise2D((int)x + 1, (int)y);
    double n3 = Noise2D((int)x, (int)y + 1);
    double n4 = Noise2D((int)x + 1, (int)y + 1);

    double i1 = cosrp(n1, n2, x - (int)x);
    double i2 = cosrp(n3, n4, x - (int)x);

    return cosrp(i1, i2, y - (int)y);
}

float PerlinNoise2D(float x, float y, float persistence, float frequency, float octaves)
{
    double total = 0.0;
    double amplitude = 1;

    for(int lcv = 0; lcv < octaves; lcv++)
    {
        total = total + SmoothNoise2D(x * frequency, y * frequency) * amplitude;
        frequency = frequency * 2;
        amplitude = amplitude * persistence;
    }

    total = MIN(MAX(total,0), 1);

    return total;
}


